//-----------------------------------------------------------------------------
// Copyright (c) 2012 GarageGames, LLC
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// DefaultTrigger is used by the mission editor.  This is also an example
// of trigger methods and callbacks.

datablock TriggerData(DefaultTrigger)
{
   // The period is value is used to control how often the console
   // onTriggerTick callback is called while there are any objects
   // in the trigger.  The default value is 100 MS.
   tickPeriodMS = 100;
};

datablock TriggerData(ClientTrigger : DefaultTrigger)
{
   clientSide = true;
};

datablock TriggerData(FlagCaptureTrigger : DefaultTrigger)
{
};
//------------------------------------------------------------------------------
// Teleport Trigger
//-----------------------------------------------------------------------------
// Teleport to a location
//
//To use add a dynamic Field with teleportLocation - (Location 0 0 0) (Rotation 1 0 0 0)
datablock TriggerData(TeleportTrigger)
{
	tickPeriodMS = 100;
};

function TeleportTrigger::onEnterTrigger(%this,%trigger,%obj)
{
   echo("Telepor Triggered");
	if(%trigger.teleportLocation !$= "")
    {
      %obj.setTransform(%trigger.teleportLocation);
    }
   %obj.setVelocity("0 0 0");
}

//Kill Trigger
datablock TriggerData(KillTrigger)
{
   tickPeriodMS = 500;
};

function KillTrigger::onEnterTrigger(%this,%trigger,%obj,%damage)
{   
    %obj.kill();
}
  
//--------------------------------------------------------------  
// Events  
//--------------------------------------------------------------  
function CapFlagTrigger::onEnterTrigger( %this, %trigger, %obj )  
{  
   // This method is called whenever an object enters the %trigger  
   // area, the object is passed as %obj.  The default onEnterTrigger  
   // method (in the C++ code) invokes the ::onTrigger(%trigger,1) method on  
   // every object (whatever it's type) in the same group as the trigger.  
   Parent::onEnterTrigger( %this, %trigger, %obj );  
   //SO the trigger requires 3 dynamic variable when placing  
   //invcheckitem  = Flag1 OR Flag2 OR whatever it is looking for in the players  
   //inv and  
   //team = 1 or 2 or whatever you labled the team property  
   // and  
   //respawnat = the name of a marker object  
   echo("ENTERING A CapFLAG TRIGGER");  
   // check to see if the lad who is in this trigger is on the our team  
   if( %obj.client.team $= %trigger.team )  
   {  
    echo("This is your team");  
        // ok, its one of our lads.. see if he has the enemy flag?  
        if (%obj.getInventory(%trigger.invcheckitem) > 0)  
        {  
            echo("You Scored");  
            // cool.. we got a capture!  
            $Game::TeamFlagCaptures[%obj.client.team]++;  
            // make a hoot about capping.  
            messageAll( 'MsgObjective', '\c2Team %1 captured a flag! %2 points!', %obj.client.team,$Game::PointsForCapture );  
            $Game::TeamScores[%obj.client.team] += $Game::PointsForCapture;  
            // take the flag out the guys inventory.  
            %obj.setInventory(%trigger.invcheckitem,0);  
            %obj.unmountImage($FlagSlot);  
            // now return the flag..  
            echo("ENTERING A CapFLAG TRIGGER");  
            newflag(%trigger.invcheckitem,%trigger.respawnat);  
  
        }  
    }  
}  