//-----------------------------------------------------------------------------
// Copyright (c) 2012 GarageGames, LLC
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//-----------------------------------------------------------------------------

//--------------------------------------------------------------------------
// Sounds
//--------------------------------------------------------------------------

datablock SFXProfile(GrenadeFireSound)
{
   filename = "art/sound/throw";
   description = BulletFireDesc;
   preload = true;
};

datablock SFXProfile(GrenadeExplodeSound)
{
   filename = "art/sound/weapons/grenade_explosion";
   description = BulletFireDesc;
   preload = true;
};

datablock SFXProfile(GrenadeSwitchinSound)
{
   filename = "art/sound/weapons/Weapon_switch";
   description = AudioClose3D;
   preload = true;
};

// ----------------------------------------------------------------------------
// Particles
// ----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Explosion
//-----------------------------------------------------------------------------
datablock ExplosionData(GrenadeDirtExplosion)
{
   soundProfile = GrenadeExplodeSound;
   lifeTimeMS = 75;

   // Volume particles
   particleEmitter = BulletDirtDustEmitter;
   particleDensity = 4;
   particleRadius = 0.3;

   // Point emission
   emitter[0] = BulletDirtSprayEmitter;
   emitter[1] = BulletDirtSprayEmitter;
   emitter[2] = BulletDirtRocksEmitter;
};

//--------------------------------------------------------------------------
// Shell ejected during reload.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Projectile Object
//-----------------------------------------------------------------------------

datablock ProjectileData( GrenadeProjectile )
{
   projectileShapeName = "art/shapes/weapons/grenades/grenade.dts";

   directDamage        = 0;
   radiusDamage        = 30;
   damageRadius        = 5;
   areaImpulse         = 2000;
   impactForce         = 2000;

   explosion           = GrenadeDirtExplosion;
   decal               = ScorchRXDecal;

   muzzleVelocity      = 20;
   velInheritFactor    = 1;

   armingDelay         = 1500;
   lifetime            = 6000;
   fadeDelay           = 1750;
   bounceElasticity    = 0.4;
   bounceFriction      = 0.1;
   isBallistic         = true;
   gravityMod          = 1;
};

//-----------------------------------------------------------------------------
// Ammo Item
//-----------------------------------------------------------------------------
datablock ItemData(GrenadeClip)
{
   // Mission editor category
   category = "AmmoClip";

   // Add the Ammo namespace as a parent.  The ammo namespace provides
   // common ammo related functions and hooks into the inventory system.
   className = "AmmoClip";

   // Basic Item properties
   shapeFile = "art/shapes/weapons/Grenades/box.DAE";
   mass = 1;
   elasticity = 0.2;
   friction = 0.6;

   // Dynamic properties defined by the scripts
   pickUpName = "Grenade clip";
   count = 1;
   maxInventory = 0;
};

datablock ItemData(GrenadeAmmo)
{
   // Mission editor category
   category = "Ammo";

   // Add the Ammo namespace as a parent.  The ammo namespace provides
   // common ammo related functions and hooks into the inventory system.
   className = "Ammo";

   // Basic Item properties
   shapeFile = "art/shapes/weapons/Grenades/box.DAE";
   mass = 1;
   elasticity = 0.2;
   friction = 0.6;

   // Dynamic properties defined by the scripts
   pickUpName = "Grenade bullet";
   maxInventory = 4;
   clip = GrenadeClip;
};

//--------------------------------------------------------------------------
// Weapon Item.  This is the item that exists in the world, i.e. when it's
// been dropped, thrown or is acting as re-spawnable item.  When the weapon
// is mounted onto a shape, the SoldierWeaponImage is used.
//-----------------------------------------------------------------------------
datablock ItemData(Grenade)
{
   // Mission editor category
   category = "Weapon";

   // Hook into Item Weapon class hierarchy. The weapon namespace
   // provides common weapon handling functions in addition to hooks
   // into the inventory system.

   // Basic Item properties
   shapeFile = "art/shapes/weapons/Grenades/box.DAE";
   mass = 1;
   elasticity = "0.199413";
   friction = "0.599218";
   emap = 1;

   // Dynamic properties defined by the scripts
   pickUpName = "Grenade";
   description = "Grenade";
   image = GrenadeWeaponImage;
   
   // weaponHUD
   PreviewImage = 'Grenade.png';
   reticle = 'blank';
   zoomReticle = 'blank';
   zoomReticleCrosshire = 'blank';
   cameraMaxDist = "0.892997";
   firstPersonOnly = "1";
   class = "Weapon";
   useEyePoint = "1";
};


datablock ShapeBaseImageData(GrenadeWeaponImage)
{
   // Basic Item properties
   shapeFile = "art/shapes/weapons/EmptyWeapon.DAE";
   shapeFileFP = "art/shapes/weapons/EmptyWeapon.DAE";
   emap = true;
   
   customZoom = 65.0; 

   imageAnimPrefix = "Grenade";
   //imageAnimPrefixFP = "Grenade";

   // Specify mount point & offset for 3rd person, and eye offset
   // for first person rendering.
   mountPoint = 1;
   firstPerson = false;
   useEyeNode = true;
   animateOnServer = true;
   eyeOffset = "0.1 0 -0.1";
   //Offset = "0.1 0.1 3";
   
   jumpingDisallowed = false;
   sprintDisallowed = false;
   crouchDisallowed = false;
   proneDisallowed = true;
   swimmingDisallowed = false;

   // When firing from a point offset from the eye, muzzle correction
   // will adjust the muzzle vector to point to the eye LOS point.
   // Since this weapon doesn't actually fire from the muzzle point,
   // we need to turn this off.
   correctMuzzleVector = true;

   // Add the WeaponImage namespace as a parent, WeaponImage namespace
   // provides some hooks into the inventory system.
   class = "WeaponImage";
   className = "WeaponImage";

   // Projectiles and Ammo.
   item = Grenade;
   ammo = GrenadeAmmo;
   clip = GrenadeClip;

   projectile = GrenadeProjectile;
   projectileType = Projectile;
   projectileSpread = "0.0";

   //altProjectile = GrenadeLauncherProjectile;
   //altProjectileSpread = "0.02";

   // Weapon lights up while firing
   lightType = "WeaponFireLight";
   lightColor = "0.992126 0.968504 0.700787 1";
   lightRadius = "4";
   lightDuration = "100";
   lightBrightness = 2;

   // Shake camera while firing.
   shakeCamera = false;
   camShakeFreq = "0 0 0";
   camShakeAmp = "0 0 0";

   // Images have a state system which controls how the animations
   // are run, which sounds are played, script callbacks, etc. This
   // state system is downloaded to the client so that clients can
   // predict state changes and animate accordingly.  The following
   // system supports basic ready->fire->reload transitions as
   // well as a no-ammo->dryfire idle state.

   useRemainderDT = true;

   // Initial start up state
   stateName[0]                     = "Preactivate";
   stateTransitionOnLoaded[0]       = "Activate";
   stateTransitionOnNoAmmo[0]       = "NoAmmo";

   // Activating the gun.  Called when the weapon is first
   // mounted and there is ammo.
   stateName[1]                     = "Activate";
   stateTransitionGeneric0In[1]     = "SprintEnter";
   stateTransitionOnTimeout[1]      = "Ready";
   stateTimeoutValue[1]             = 0.1;
   stateSequence[1]                 = "switch_in";

   // Ready to fire, just waiting for the trigger
   stateName[2]                     = "Ready";
   stateTransitionGeneric0In[2]     = "SprintEnter";
   stateTransitionOnMotion[2]       = "ReadyMotion";
   stateTransitionOnTimeout[2]      = "ReadyFidget";
   stateTimeoutValue[2]             = 10;
   stateWaitForTimeout[2]           = false;
   stateScaleAnimation[2]           = false;
   stateScaleAnimationFP[2]         = false;
   stateTransitionOnNoAmmo[2]       = "NoAmmo";
   stateTransitionOnTriggerDown[2]  = "Fire";
   stateSequence[2]                 = "idle";

   // Same as Ready state but plays a fidget sequence
   stateName[3]                     = "ReadyFidget";
   stateTransitionGeneric0In[3]     = "SprintEnter";
   stateTransitionOnMotion[3]       = "ReadyMotion";
   stateTransitionOnTimeout[3]      = "Ready";
   stateTimeoutValue[3]             = 6;
   stateWaitForTimeout[3]           = false;
   stateTransitionOnNoAmmo[3]       = "NoAmmo";
   stateTransitionOnTriggerDown[3]  = "Fire";
   stateSequence[3]                 = "idle_fidget1";

   // Ready to fire with player moving
   stateName[4]                     = "ReadyMotion";
   stateTransitionGeneric0In[4]     = "SprintEnter";
   stateTransitionOnNoMotion[4]     = "Ready";
   stateWaitForTimeout[4]           = false;
   stateScaleAnimation[4]           = false;
   stateScaleAnimationFP[4]         = false;
   stateSequenceTransitionIn[4]     = true;
   stateSequenceTransitionOut[4]    = true;
   stateTransitionOnNoAmmo[4]       = "NoAmmo";
   stateTransitionOnTriggerDown[4]  = "Fire";
   stateSequence[4]                 = "run";

   // Fire the weapon. Calls the fire script which does
   // the actual work.
   stateName[5]                     = "Fire";
   stateTransitionGeneric0In[5]     = "SprintEnter";
   stateTransitionOnTimeout[5]      = "Reload";
   stateTimeoutValue[5]             = 1;
   stateFire[5]                     = true;
   stateRecoil[5]                   = "";
   stateAllowImageChange[5]         = false;
   stateSequence[5]                 = "fire";
   stateScaleAnimation[5]           = false;
   stateSequenceNeverTransition[5]  = true;
   stateSequenceRandomFlash[5]      = true;        // use muzzle flash sequence
   stateScript[5]                   = "onFire";
   stateSound[5]                    = GrenadeFireSound;
   stateEmitter[5]                  = GunFireSmokeEmitter;
   stateEmitterTime[5]              = 0.025;
   
   // Play the reload animation, and transition into
   stateName[6]                     = "Reload";
   stateTransitionGeneric0In[6]     = "SprintEnter";
   stateTransitionOnNoAmmo[6]       = "NoAmmo";
   stateTransitionOnTimeout[6]      = "Ready";
   stateWaitForTimeout[6]           = "0";
   stateTimeoutValue[6]             = 1;
   stateAllowImageChange[6]         = false;
   stateSequence[6]                 = "reload";
   stateEjectShell[6]               = true;


   // No ammo in the weapon, just idle until something
   // shows up. Play the dry fire sound if the trigger is
   // pulled.
   stateName[7]                     = "NoAmmo";
   stateTransitionGeneric0In[7]     = "SprintEnter";
   stateTransitionOnAmmo[7]         = "ReloadClip";
   stateScript[7]                   = "onClipEmpty";

   // No ammo dry fire
   stateName[8]                     = "DryFire";
   stateTransitionGeneric0In[8]     = "SprintEnter";
   stateTimeoutValue[8]             = 1.0;
   stateTransitionOnTimeout[8]      = "NoAmmo";
   stateScript[8]                   = "onDryFire";


   // Play the reload clip animation
   stateName[9]                     = "ReloadClip";
   stateTransitionGeneric0In[9]     = "SprintEnter";
   stateTransitionOnTimeout[9]      = "Ready";
   stateWaitForTimeout[9]           = true;
   stateTimeoutValue[9]             = 2.0;
   stateReload[9]                   = true;
   stateSequence[9]                 = "reload";
   stateSound[9]                    = GrenadeReloadSound;
   stateShapeSequence[9]            = "Reload";
   stateScaleShapeSequence[9]       = true;


   // Start Sprinting
   stateName[10]                    = "SprintEnter";
   stateTransitionGeneric0Out[10]   = "SprintExit";
   stateTransitionOnTimeout[10]     = "Sprinting";
   stateWaitForTimeout[10]          = false;
   stateTimeoutValue[10]            = 0.5;
   stateWaitForTimeout[10]          = false;
   stateScaleAnimation[10]          = false;
   stateScaleAnimationFP[10]        = false;
   stateSequenceTransitionIn[10]    = true;
   stateSequenceTransitionOut[10]   = true;
   stateAllowImageChange[10]        = false;
   stateSequence[10]                = "sprint";

   // Sprinting
   stateName[11]                    = "Sprinting";
   stateTransitionGeneric0Out[11]   = "SprintExit";
   stateWaitForTimeout[11]          = false;
   stateScaleAnimation[11]          = false;
   stateScaleAnimationFP[11]        = false;
   stateSequenceTransitionIn[11]    = true;
   stateSequenceTransitionOut[11]   = true;
   stateAllowImageChange[11]        = false;
   stateSequence[11]                = "sprint";
   
   // Stop Sprinting
   stateName[12]                    = "SprintExit";
   stateTransitionGeneric0In[12]    = "SprintEnter";
   stateTransitionOnTimeout[12]     = "Ready";
   stateWaitForTimeout[12]          = false;
   stateTimeoutValue[12]            = 0.5;
   stateSequenceTransitionIn[12]    = true;
   stateSequenceTransitionOut[12]   = true;
   stateAllowImageChange[12]        = false;
   stateSequence[12]                = "sprint";
};

function GrenadeWeaponImage::onDamage(%this, %obj)
{
   // Images assume a false ammo state on load.  We need to
   // set the state according to the current inventory.
   if(%this.isField("clip"))
   {
      // Use the clip system for this weapon.  Check if the player already has
      // some ammo in a clip.
      if (%obj.getInventory(%this.ammo))
      {
         %obj.setImageAmmo(%slot, true);
         %currentAmmo = %obj.getInventory(%this.ammo);
      }
      else if(%obj.getInventory(%this.clip) > 0)
      {
         // Fill the weapon up from the first clip
         %obj.setInventory(%this.ammo, %this.ammo.maxInventory);
         %obj.setImageAmmo(%slot, true);
         
         // Add any spare ammo that may be "in the player's pocket"
         %currentAmmo = %this.ammo.maxInventory;
         %amountInClips += %obj.getFieldValue( "remaining" @ %this.ammo.getName());
      }
      else
      {
         %currentAmmo = 0 + %obj.getFieldValue( "remaining" @ %this.ammo.getName());
      }
      
      %amountInClips = %obj.getInventory(%this.clip);
      %amountInClips *= %this.ammo.maxInventory;
      
      if (%obj.client !$= "" && !%obj.isAiControlled)
         %obj.client.RefreshWeaponHud(%currentAmmo, %this.item.previewImage, %this.item.reticle, %this.item.zoomReticle, %amountInClips);
   }
   else if(%this.ammo !$= "")
   {
      // Use the ammo pool system for this weapon
      if (%obj.getInventory(%this.ammo))
      {
         %obj.setImageAmmo(%slot, true);
         %currentAmmo = %obj.getInventory(%this.ammo);
      }
      else
         %currentAmmo = 0;

      if (%obj.client !$= "" && !%obj.isAiControlled)
         %obj.client.RefreshWeaponHud( 1, %this.item.previewImage, %this.item.reticle, %this.item.zoomReticle, %currentAmmo );
   }
}

function GrenadeWeaponImage::reloadAmmoClip(%this, %obj, %slot)
{
   //Parent::onUnMount(%this, %obj, %slot);

   // Make sure we're indeed the currect image on the given slot
   if (%this != %obj.getMountedImage(%slot))
      return;

   if ( %this.isField("clip") )
   {
      if (%obj.getInventory(%this.clip) > 0)
      {
         %obj.decInventory(%this.clip, 1);
         %obj.setInventory(%this.ammo, %this.ammo.maxInventory);
         %obj.setImageAmmo(%slot, true);
      }
      else
      {
         %amountInPocket = %obj.getFieldValue( "remaining" @ %this.ammo.getName());
         if ( %amountInPocket )
         {
            %obj.setFieldValue( "remaining" @ %this.ammo.getName(), 0);
            %obj.setInventory( %this.ammo, %amountInPocket );
            %obj.setImageAmmo( %slot, true );
         }
      }
      
   }
}

function GrenadeWeaponImage::pickup(%this, %obj, %amount)
{
   // This method is called to pickup an object and add it to the inventory.
   // The datablock onPickup method is actually responsible for doing all the
   // work, including incrementing the inventory.
   %data = %obj.getDatablock();

   // Try and pickup the max if no value was specified
   if (%amount $= "")
      %amount = %this.maxInventory(%data) - %this.getInventory(%data);

   // The datablock does the work...
   if (%amount < 0)
      %amount = 0;
   if (%amount)
      return %data.onPickup(%obj, %this, %amount);
   return false;
   
}