//-----------------------------------------------------------------------------
// Copyright (c) 2012 GarageGames, LLC
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//-----------------------------------------------------------------------------

//--------------------------------------------------------------------------
// Sounds
//--------------------------------------------------------------------------

datablock SFXProfile(KnivesFireSound)
{
   filename = "art/sound/weapons/Knives_Fire";
   description = BulletFireDesc;
   preload = true;
};

datablock SFXProfile(KnivesReloadSound)
{
   filename = "art/sound/weapons/Weapon_reload";
   description = AudioClose3D;
   preload = true;
};

datablock SFXProfile(KnivesSwitchinSound)
{
   filename = "art/sound/weapons/Weapon_switch";
   description = AudioClose3D;
   preload = true;
};

datablock SFXProfile(SniperRiflBulletImpactSound)
{
   filename = "art/sound/weapons/Bullet_impact1";
   description = AudioClose3D;
   preload = true;
};


// ----------------------------------------------------------------------------
// Particles
// ----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Explosion
//-----------------------------------------------------------------------------
datablock ExplosionData(SniperBulletDirtExplosion)
{
   soundProfile = SniperRiflBulletImpactSound;
   lifeTimeMS = 75;

   // Volume particles
   particleEmitter = BulletDirtDustEmitter;
   particleDensity = 4;
   particleRadius = 0.3;

   // Point emission
   emitter[0] = BulletDirtSprayEmitter;
   emitter[1] = BulletDirtSprayEmitter;
   emitter[2] = BulletDirtRocksEmitter;
};

//--------------------------------------------------------------------------
// Shell ejected during reload.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Projectile Object
//-----------------------------------------------------------------------------

datablock ProjectileData( SniperBulletProjectile )
{
   projectileShapeName = "art/shapes/weapons/misc/bullet.dts";

   directDamage        = 20;
   radiusDamage        = 0;
   damageRadius        = 0.5;
   areaImpulse         = 0.5;
   impactForce         = 2;

   explosion           = BulletDirtExplosion;
   decal               = BulletHoleDecal;

   muzzleVelocity      = 1000;
   velInheritFactor    = 0.1;

   armingDelay         = 0;
   lifetime            = 3000;
   fadeDelay           = 1472;
   bounceElasticity    = 0;
   bounceFriction      = 0;
   isBallistic         = false;
   gravityMod          = 1;
};


//-----------------------------------------------------------------------------
// Ammo Item
//-----------------------------------------------------------------------------
datablock ItemData(KnivesClip)
{
   // Mission editor category
   category = "AmmoClip";

   // Add the Ammo namespace as a parent.  The ammo namespace provides
   // common ammo related functions and hooks into the inventory system.
   className = "AmmoClip";

   // Basic Item properties
   shapeFile = "art/shapes/weapons/Knives/Knives.DAE";
   mass = 1;
   elasticity = 0.2;
   friction = 0.6;

   // Dynamic properties defined by the scripts
   pickUpName = "Knives clip";
   count = 1;
   maxInventory = 1;
};

datablock ItemData(KnivesAmmo)
{
   // Mission editor category
   category = "Ammo";

   // Add the Ammo namespace as a parent.  The ammo namespace provides
   // common ammo related functions and hooks into the inventory system.
   className = "Ammo";

   // Basic Item properties
   shapeFile = "art/shapes/weapons/Knives/Knives.DAE";
   mass = 1;
   elasticity = 0.2;
   friction = 0.6;

   // Dynamic properties defined by the scripts
   pickUpName = "Knives bullet";
   maxInventory = 1;
   clip = KnivesClip;
};

//--------------------------------------------------------------------------
// Weapon Item.  This is the item that exists in the world, i.e. when it's
// been dropped, thrown or is acting as re-spawnable item.  When the weapon
// is mounted onto a shape, the SoldierWeaponImage is used.
//-----------------------------------------------------------------------------
datablock ItemData(Knives)
{
   // Mission editor category
   category = "Weapon";

   // Hook into Item Weapon class hierarchy. The weapon namespace
   // provides common weapon handling functions in addition to hooks
   // into the inventory system.
   className = "Weapon";

   // Basic Item properties
   shapeFile = "art/shapes/weapons/Knives/Knives.DAE";
   mass = 1;
   elasticity = 0.2;
   friction = 0.6;
   emap = true;

   // Dynamic properties defined by the scripts
   pickUpName = "Knives";
   description = "Knives";
   image = KnivesWeaponImage;
   
   // weaponHUD
   PreviewImage = 'Knives.png';
   reticle = 'blank';
   zoomReticle = 'bino';
   zoomReticleCrosshire = 'reticle_sniper';
   
   
   //Set the max distance a bot will fire this weapon from
   ignoreDistance = 500;
   //Set the min distance a bot will fire this weapon from
   minIgnoreDistance = 0;
   //Set if this weapon uses ammo or not
   usesAmmo = false;
   //Sets how long the bot waits between firing bursts
   fireDelay = 1000;
   //Sets how long the bot holds down the trigger when firing. Use longer pulses for spray and pray type weapons.
   triggerDown = 500;
   //Sets the relative quality score of the weapon, how good is the weapon?
   //This should be set to a different value for every weapon.
   weapRating = 5;
   //The amount of ammo that bots using this weapon should start with.
   weapStartAmmo = 1;
   //Enemies using this weapon are unable to aim or move while firing it.
   notAim = true;
};


datablock ShapeBaseImageData(KnivesWeaponImage)
{
   // Basic Item properties
   shapeFile = "art/shapes/weapons/Knives/Knives_TP.DAE";
   shapeFileFP = "art/shapes/weapons/EmptyWeapon.DAE";
   emap = true;

   imageAnimPrefix = "Knives";
   //imageAnimPrefixFP = "Knives";

   // Specify mount point & offset for 3rd person, and eye offset
   // for first person rendering.
   mountPoint = 0;
   firstPerson = false;
   useEyeNode = true;
   animateOnServer = true;
   eyeOffset = "0 0 -0.1";
      
   customZoom = 5.0; 
   
   jumpingDisallowed = true;
   sprintDisallowed = true;
   crouchDisallowed = true;
   proneDisallowed = true;
   swimmingDisallowed = true;

   // When firing from a point offset from the eye, muzzle correction
   // will adjust the muzzle vector to point to the eye LOS point.
   // Since this weapon doesn't actually fire from the muzzle point,
   // we need to turn this off.
   correctMuzzleVector = true;

   // Add the WeaponImage namespace as a parent, WeaponImage namespace
   // provides some hooks into the inventory system.
   class = "WeaponImage";
   className = "WeaponImage";

   // Projectiles and Ammo.
   item = Knives;
   ammo = KnivesAmmo;
   clip = KnivesClip;

   projectile = SniperBulletProjectile;
   projectileType = Projectile;
   projectileSpread = "0.0";

   casing = BulletShell;
   shellExitDir        = "1.0 0.3 1.0";
   shellExitOffset     = "0.15 -0.56 -0.1";
   shellExitVariance   = 15.0;
   shellVelocity       = 3.0;

   // Weapon lights up while firing
   lightType = "WeaponFireLight";
   lightColor = "0.992126 0.968504 0.700787 1";
   lightRadius = "4";
   lightDuration = "100";
   lightBrightness = 2;

   // Shake camera while firing.
   shakeCamera = false;
   camShakeFreq = "0 0 0";
   camShakeAmp = "0 0 0";

   // Images have a state system which controls how the animations
   // are run, which sounds are played, script callbacks, etc. This
   // state system is downloaded to the client so that clients can
   // predict state changes and animate accordingly.  The following
   // system supports basic ready->fire->reload transitions as
   // well as a no-ammo->dryfire idle state.

   useRemainderDT = true;

   // Initial start up state
   stateName[0]                     = "Preactivate";
   stateTransitionOnLoaded[0]       = "Activate";

   // Activating the gun.  Called when the weapon is first
   // mounted and there is ammo.
   stateName[1]                     = "Activate";
   stateTransitionOnTimeout[1]      = "Ready";
   stateTimeoutValue[1]             = 0.5;  // 0.5 seconds until the weapon is usable
   stateSequence[1]                 = "switch_in";
   stateSound[1]                    = KnivesSwitchinSound;

   // Ready to fire, just waiting for the trigger
   stateName[2]                     = "Ready";
   stateTimeoutValue[2]             = 10;
   stateWaitForTimeout[2]           = false;
   stateScaleAnimation[2]           = false;
   stateScaleAnimationFP[2]         = false;
   //stateTransitionOnNoAmmo[2]       = "NoAmmo";
   stateTransitionOnTriggerDown[2]  = "Fire";
   stateSequence[2]                 = "idle";

   // Fire the weapon. Calls the fire script which does
   // the actual work.
   stateName[3]                     = "Fire";
   stateTransitionOnTimeout[3]      = "Reload";
   stateTimeoutValue[3]             = 0.15;
   stateFire[3]                     = true;
   stateRecoil[3]                   = "";
   stateAllowImageChange[3]         = false;
   stateSequence[3]                 = "fire";
   stateScaleAnimation[3]           = false;
   stateSequenceNeverTransition[3]  = true;
   stateSequenceRandomFlash[3]      = true;        // use muzzle flash sequence
   stateScript[3]                   = "onFire";
   stateSound[3]                    = KnivesFireSound;
   stateEmitter[3]                  = GunFireSmokeEmitter;
   stateEmitterTime[3]              = 0.025;

   // Play the reload animation, and transition into
   stateName[4]                     = "Reload";
   stateTransitionOnTimeout[4]      = "Ready";
   stateWaitForTimeout[4]           = "0";
   stateTimeoutValue[4]             = 2;  //Reloading Takes 2 seconds
   stateAllowImageChange[4]         = false;
   stateSequence[4]                 = "reload";
   stateSound[4]                    = KnivesReloadSound;
   stateEjectShell[4]               = true;
};