//-----------------------------------------------------------------------------
// Copyright (c) 2012 GarageGames, LLC
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//-----------------------------------------------------------------------------

//--------------------------------------------------------------------------
// Sounds
//--------------------------------------------------------------------------

datablock SFXProfile(UziFireSound)
{
   filename = "art/sound/weapons/Uzi_Fire";
   description = BulletFireDesc;
   preload = true;
};

datablock SFXProfile(UziReloadSound)
{
   filename = "art/sound/weapons/Weapon_reload";
   description = AudioClose3D;
   preload = true;
};

datablock SFXProfile(UziSwitchinSound)
{
   filename = "art/sound/weapons/Weapon_switch";
   description = AudioClose3D;
   preload = true;
};

// ----------------------------------------------------------------------------
// Particles
// ----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Explosion
//-----------------------------------------------------------------------------

//--------------------------------------------------------------------------
// Shell ejected during reload.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Projectile Object
//-----------------------------------------------------------------------------
datablock ProjectileData( UziBulletProjectile )
{
   projectileShapeName = "art/shapes/weapons/misc/bullet.dts";

   directDamage        = 4;
   radiusDamage        = 0;
   damageRadius        = 0.5;
   areaImpulse         = 0.5;
   impactForce         = 1;

   explosion           = BulletDirtExplosion;
   decal               = BulletHoleDecal;

   muzzleVelocity      = 80;
   velInheritFactor    = 0;

   armingDelay         = 0;
   lifetime            = 3000;
   fadeDelay           = 1472;
   bounceElasticity    = 0;
   bounceFriction      = 0;
   isBallistic         = false;
   gravityMod          = 1;
   
   sound = "BulletBuzz5Sound";
};

//-----------------------------------------------------------------------------
// Ammo Item
//-----------------------------------------------------------------------------
datablock ItemData(UziClip)
{
   // Mission editor category
   category = "AmmoClip";

   // Add the Ammo namespace as a parent.  The ammo namespace provides
   // common ammo related functions and hooks into the inventory system.
   className = "AmmoClip";

   // Basic Item properties
   shapeFile = "art/shapes/weapons/Uzi/Uzi.DAE";
   mass = 1;
   elasticity = 0.2;
   friction = 0.6;

   // Dynamic properties defined by the scripts
   pickUpName = "Uzi clip";
   count = 1;
   maxInventory = 1;
};

datablock ItemData(UziAmmo)
{
   // Mission editor category
   category = "Ammo";

   // Add the Ammo namespace as a parent.  The ammo namespace provides
   // common ammo related functions and hooks into the inventory system.
   className = "Ammo";

   // Basic Item properties
   shapeFile = "art/shapes/weapons/Uzi/Uzi.DAE";
   mass = 1;
   elasticity = 0.2;
   friction = 0.6;

   // Dynamic properties defined by the scripts
   pickUpName = "Uzi bullet";
   maxInventory = 24;
   clip = UziClip;
};

//--------------------------------------------------------------------------
// Weapon Item.  This is the item that exists in the world, i.e. when it's
// been dropped, thrown or is acting as re-spawnable item.  When the weapon
// is mounted onto a shape, the SoldierWeaponImage is used.
//-----------------------------------------------------------------------------
datablock ItemData(Uzi)
{
   // Mission editor category
   category = "Weapon";

   // Hook into Item Weapon class hierarchy. The weapon namespace
   // provides common weapon handling functions in addition to hooks
   // into the inventory system.

   // Basic Item properties
   shapeFile = "art/shapes/weapons/Uzi/Uzi.DAE";
   mass = 1;
   elasticity = "0.2";
   friction = "0.6";
   emap = 1;

   imageAnimPrefix = "Uzi";
   imageAnimPrefixFP = "Uzi";

   // Dynamic properties defined by the scripts
   pickUpName = "Uzi";
   description = "Uzi";
   image = UziWeaponImage;
   
   // weaponHUD
   PreviewImage = 'Uzi.png';
   reticle = 'reticle_crosshair';
   zoomReticle = 'blank';
   zoomReticleCrosshire = 'reticle_crosshair';
   //cameraMaxDist = "0.892997";
   firstPersonOnly = "1";
   class = "Weapon";
   useEyePoint = "1";
   
   //Set the max distance a bot will fire this weapon from
   ignoreDistance = 50;
   //Set the min distance a bot will fire this weapon from
   minIgnoreDistance = 0;
   //Set if this weapon uses ammo or not
   usesAmmo = true;
   //Sets how long the bot waits between firing bursts
   fireDelay = 1000;
   //Sets how long the bot holds down the trigger when firing. Use longer pulses for spray and pray type weapons.
   triggerDown = 500;
   //Sets the relative quality score of the weapon, how good is the weapon?
   //This should be set to a different value for every weapon.
   weapRating = 5;
   //The amount of ammo that bots using this weapon should start with.
   weapStartAmmo = 24;
   //Enemies using this weapon are unable to aim or move while firing it.
   notAim = false;
};


datablock ShapeBaseImageData(UziWeaponImage)
{
   // Basic Item properties
   shapeFile = "art/shapes/weapons/Uzi/Uzi.DAE";
   shapeFileFP = "art/shapes/weapons/EmptyWeapon.DAE";
   emap = true;
   
   customZoom = 65.0; 

   imageAnimPrefix = "Uzi";
   imageAnimPrefixFP = "Uzi";

   // Specify mount point & offset for 3rd person, and eye offset
   // for first person rendering.
   mountPoint = 0;
   firstPerson = false;
   useEyeNode = false;
   animateOnServer = true;
   eyeOffset = "0 0 0";
   Offset = "0 0 0";
   
   jumpingDisallowed = true;
   sprintDisallowed = true;
   crouchDisallowed = true;
   proneDisallowed = true;
   swimmingDisallowed = true;
   //infiniteAmmo = true;

   // When firing from a point offset from the eye, muzzle correction
   // will adjust the muzzle vector to point to the eye LOS point.
   // Since this weapon doesn't actually fire from the muzzle point,
   // we need to turn this off.
   correctMuzzleVector = true;

   // Add the WeaponImage namespace as a parent, WeaponImage namespace
   // provides some hooks into the inventory system.
   class = "WeaponImage";
   className = "WeaponImage";

   // Projectiles and Ammo.
   item = Uzi;
   ammo = UziAmmo;
   clip = UziClip;

   projectile = UziBulletProjectile;
   projectileType = Projectile;
   projectileSpread = "0.01";

   //altProjectile = GrenadeLauncherProjectile;
   //altProjectileSpread = "0.02";

   casing = BulletShell;
   shellExitDir        = "1.0 0.3 1.0";
   shellExitOffset     = "0.15 -0.56 -0.1";
   shellExitVariance   = 15.0;
   shellVelocity       = 3.0;

   // Weapon lights up while firing
   lightType = "WeaponFireLight";
   lightColor = "0.992126 0.968504 0.700787 1";
   lightRadius = "4";
   lightDuration = "100";
   lightBrightness = 2;

   // Shake camera while firing.
   shakeCamera = false;
   camShakeFreq = "0 0 0";
   camShakeAmp = "0 0 0";

   // Images have a state system which controls how the animations
   // are run, which sounds are played, script callbacks, etc. This
   // state system is downloaded to the client so that clients can
   // predict state changes and animate accordingly.  The following
   // system supports basic ready->fire->reload transitions as
   // well as a no-ammo->dryfire idle state.

   useRemainderDT = true;

   // Initial start up state
   stateName[0]                     = "Preactivate";
   stateTransitionOnLoaded[0]       = "Activate";
   stateTransitionOnNoAmmo[0]       = "NoAmmo";

   // Activating the gun.  Called when the weapon is first
   // mounted and there is ammo.
   stateName[1]                     = "Activate";
   stateTransitionGeneric0In[1]     = "SprintEnter";
   stateTransitionOnTimeout[1]      = "Ready";
   stateTimeoutValue[1]             = 0.1;
   stateSequence[1]                 = "switch_in";

   // Ready to fire, just waiting for the trigger
   stateName[2]                     = "Ready";
   stateTransitionGeneric0In[2]     = "SprintEnter";
   stateTransitionOnMotion[2]       = "ReadyMotion";
   stateTransitionOnTimeout[2]      = "ReadyFidget";
   stateTimeoutValue[2]             = 10;
   stateWaitForTimeout[2]           = false;
   stateScaleAnimation[2]           = false;
   stateScaleAnimationFP[2]         = false;
   stateTransitionOnNoAmmo[2]       = "NoAmmo";
   stateTransitionOnTriggerDown[2]  = "Fire";
   stateSequence[2]                 = "idle";

   // Same as Ready state but plays a fidget sequence
   stateName[3]                     = "ReadyFidget";
   stateTransitionGeneric0In[3]     = "SprintEnter";
   stateTransitionOnMotion[3]       = "ReadyMotion";
   stateTransitionOnTimeout[3]      = "Ready";
   stateTimeoutValue[3]             = 6;
   stateWaitForTimeout[3]           = false;
   stateTransitionOnNoAmmo[3]       = "NoAmmo";
   stateTransitionOnTriggerDown[3]  = "Fire";
   stateSequence[3]                 = "idle_fidget1";

   // Ready to fire with player moving
   stateName[4]                     = "ReadyMotion";
   stateTransitionGeneric0In[4]     = "SprintEnter";
   stateTransitionOnNoMotion[4]     = "Ready";
   stateWaitForTimeout[4]           = false;
   stateScaleAnimation[4]           = false;
   stateScaleAnimationFP[4]         = false;
   stateSequenceTransitionIn[4]     = true;
   stateSequenceTransitionOut[4]    = true;
   stateTransitionOnNoAmmo[4]       = "NoAmmo";
   stateTransitionOnTriggerDown[4]  = "Fire";
   stateSequence[4]                 = "run";

   // Fire the weapon. Calls the fire script which does
   // the actual work.
   stateName[5]                     = "Fire";
   stateTransitionGeneric0In[5]     = "SprintEnter";
   stateTransitionOnTimeout[5]      = "Reload";
   stateTimeoutValue[5]             = 0.075;
   stateFire[5]                     = true;
   stateRecoil[5]                   = "";
   stateAllowImageChange[5]         = false;
   stateSequence[5]                 = "fire";
   stateScaleAnimation[5]           = false;
   stateSequenceNeverTransition[5]  = true;
   stateSequenceRandomFlash[5]      = true;        // use muzzle flash sequence
   stateScript[5]                   = "onFire";
   stateSound[5]                    = UziFireSound;
   stateEmitter[5]                  = GunFireSmokeEmitter;
   stateEmitterTime[5]              = 0.025;

   // Play the reload animation, and transition into
   stateName[6]                     = "Reload";
   stateTransitionGeneric0In[6]     = "SprintEnter";
   stateTransitionOnNoAmmo[6]       = "NoAmmo";
   stateTransitionOnTimeout[6]      = "Ready";
   stateWaitForTimeout[6]           = "0";
   stateTimeoutValue[6]             = 0.05;
   stateAllowImageChange[6]         = false;
   //stateSequence[6]                 = "reload";
   stateEjectShell[6]               = true;

   // No ammo in the weapon, just idle until something
   // shows up. Play the dry fire sound if the trigger is
   // pulled.
   stateName[7]                     = "NoAmmo";
   stateTransitionGeneric0In[7]     = "SprintEnter";
   stateTransitionOnAmmo[7]         = "ReloadClip";
   stateScript[7]                   = "onClipEmpty";

   // No ammo dry fire
   stateName[8]                     = "DryFire";
   stateTransitionGeneric0In[8]     = "SprintEnter";
   stateTimeoutValue[8]             = 1.0;
   stateTransitionOnTimeout[8]      = "NoAmmo";
   stateScript[8]                   = "onDryFire";

   // Play the reload clip animation
   stateName[9]                     = "ReloadClip";
   stateTransitionGeneric0In[9]     = "SprintEnter";
   stateTransitionOnTimeout[9]      = "Ready";
   stateWaitForTimeout[9]           = true;
   stateTimeoutValue[9]             = 2.0;
   stateReload[9]                   = true;
   stateSequence[9]                 = "reload";
   stateSound[9]                    = UziReloadSound;
   stateShapeSequence[9]            = "Reload";
   stateScaleShapeSequence[9]       = true;

   // Start Sprinting
   stateName[10]                    = "SprintEnter";
   stateTransitionGeneric0Out[10]   = "SprintExit";
   stateTransitionOnTimeout[10]     = "Sprinting";
   stateWaitForTimeout[10]          = false;
   stateTimeoutValue[10]            = 0.5;
   stateWaitForTimeout[10]          = false;
   stateScaleAnimation[10]          = false;
   stateScaleAnimationFP[10]        = false;
   stateSequenceTransitionIn[10]    = true;
   stateSequenceTransitionOut[10]   = true;
   stateAllowImageChange[10]        = false;
   stateSequence[10]                = "sprint";

   // Sprinting
   stateName[11]                    = "Sprinting";
   stateTransitionGeneric0Out[11]   = "SprintExit";
   stateWaitForTimeout[11]          = false;
   stateScaleAnimation[11]          = false;
   stateScaleAnimationFP[11]        = false;
   stateSequenceTransitionIn[11]    = true;
   stateSequenceTransitionOut[11]   = true;
   stateAllowImageChange[11]        = false;
   stateSequence[11]                = "sprint";
   
   // Stop Sprinting
   stateName[12]                    = "SprintExit";
   stateTransitionGeneric0In[12]    = "SprintEnter";
   stateTransitionOnTimeout[12]     = "Ready";
   stateWaitForTimeout[12]          = false;
   stateTimeoutValue[12]            = 0.5;
   stateSequenceTransitionIn[12]    = true;
   stateSequenceTransitionOut[12]   = true;
   stateAllowImageChange[12]        = false;
   stateSequence[12]                = "sprint";
};
datablock ItemData(UziAmmo2 : UziAmmo)
{
   //Copy
};
datablock ItemData(UziClip2 : UziClip)
{
   //Copy
};

datablock ShapeBaseImageData(Uzi2WeaponImage : UziWeaponImage)
{
   mountPoint = 1;
      // Projectiles and Ammo.
   item = Uzi;
   ammo = UziAmmo2;
   clip = UziClip2;
};

function UziWeaponImage::onMount(%this,%obj)
{
   Parent::onMount(%this,%obj);
      %obj.mountImage(Uzi2WeaponImage,1);
}

function UziWeaponImage::onDamage(%this, %obj)
{
   Parent::onDamage(%this, %obj);
}

function UziWeaponImage::onFire(%data, %obj, %slot)
{
   Parent::onFire(%data, %obj, %slot);
      %obj.setImageTrigger(1, true);
}

function Uzi2WeaponImage::onFire(%data,%obj,%slot)
{
   Parent::onFire(%data,%obj,%slot);
}

function UziWeaponImage::onUnMount(%this, %obj, %slot)
{
   Parent::onUnMount(%this, %obj, %slot);
   %obj.unMountImage(1);
}